package de.lmu.ifi.mfchords.utils;

//import java.awt.geom.AffineTransform;
//import java.awt.geom.Line2D;
//import java.awt.geom.NoninvertibleTransformException;
//import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import android.graphics.Matrix;
import android.graphics.PointF;
import android.util.Log;

public class Utils {
	
	final static int BLOCK = 0;
	final static int TRIAL = 1;
	final static int PRACTICE = 2;
	final static int TECHNIQUE = 3;
	final static int FINGERS = 4;
	final static int TIMESTAMP = 5;
	final static int EVENT = 6;
	final static int TOUCHID = 7;
	final static int TOUCH_X = 8;
	final static int TOUCH_Y = 9;
	final static int SIZE = 10;
	final static int MINOR = 11;
	final static int MAJOR = 12;
	
	
	
	public static Float scalarP(ArrayList<Float> v1, ArrayList<Float> v2){
		if(v1.size() != v2.size()){
			System.out.println("ERROR: vector of unequal size");
			return -1f;
		}
		
		Float result = 0f;
		for(int c = 0; c<v1.size(); c++){
			result = result + v1.get(c) * v2.get(c);
		}
		return result;
	}
	
	public static Double scalarP(Double[] v1, Double[] v2){
		if(v1.length != v2.length){
			System.out.println("ERROR: vector of unequal size");
			return -1.0;
		}
		
		Double result = 0.0;
		for(int c = 0; c<v1.length; c++){
			result = result + v1[c] * v2[c];
		}
		return result;
	}
	
	public static Float magnitude(ArrayList<Float> v){
		if(v == null){
			return -1f;
		}
		Float result = 0f;
		for(int c = 0; c<v.size(); c++){
			result = result + v.get(c) * v.get(c);
		}
		return Float.valueOf((float)Math.sqrt(result.doubleValue()));
	}
	
	public static Double magnitude(Double[] v){
		if(v.length == 0){
			return -1.0;
		}
		Double result = 0.0;
		for(int c = 0; c<v.length; c++){
			result = result + v[c] * v[c];
		}
		return Math.sqrt(result);
	}
	
	public static Float magnitude(Float[] v){
		if(v.length == 0){
			return -1.0f;
		}
		Float result = 0.0f;
		for(int c = 0; c<v.length; c++){
			result = result + v[c] * v[c];
		}
		return (float) Math.sqrt(Float.valueOf(result));
	}
	
	public static Double[] getUnitVector(Double[] v){
		
		if(v.length == 0){
			return null;
		}
		Double[] result = new Double[v.length];
		Double m_v = magnitude(v);
		for(int c = 0; c<v.length; c++){
			result[c] = v[c] / m_v;
		}
		return result;
	}
	
	public static float slope(float p1_x, float p1_y, float p2_x, float p2_y){
		return(p2_y - p1_y)/(p2_x-p1_x);
	}
	
	public static double slope(double p1_x, double p1_y, double p2_x, double p2_y){
		return(p2_y - p1_y)/(p2_x-p1_x);
	}
	
	public static double lengthOfLine(Line2D line){
		return distanceBetweenPoints(line.getP1().x, line.getP1().y, line.getP2().x, line.getP2().y);
	}
	
	public static double distanceBetweenPoints(double p1x, double p1y, double p2x, double p2y){
	   	return Math.sqrt((p2x - p1x)*(p2x - p1x) + (p2y - p1y)*(p2y - p1y));
	    	
	}
	
	public static double relativePosition(Line2D line, PointF point){
		double lineLength = lengthOfLine(line);
		double numerator = ((line.getP1().y - point.y) * (line.getP1().y - line.getP2().y)) - ((line.getP1().x - point.x)* (line.getP2().x - line.getP1().x)); 
		//float numerator = (me.getY(thumbIndex) - me.getY(thirdIndex)) * (me.getY(thumbIndex) - me.getY(pinkyIndex)) - (me.getX(thumbIndex) - me.getX(thirdIndex)) * (me.getX(pinkyIndex)-me.getX(thumbIndex));
    	return numerator / (lineLength * lineLength);
	}
	
	
	
	public static double computeAngle (double[] p0, double[] p1, double[] p2, double[] p3)
	{
	  double[] v0 = Geometry.createVector (p0, p1);
	  double[] v1 = Geometry.createVector (p2, p3);

	  //System.out.println("v0 x" + v0[0] + " y " + v0[1]);
	  //System.out.println("v1 x" + v1[0] + " y " + v1[1]);
	  
	  double dotProduct = Geometry.computeDotProduct (v0, v1);
	  //System.out.println("dot product " + dotProduct);

	  double length1 = Geometry.length (v0);
	  //System.out.println("length " + length1);
	  double length2 = Geometry.length (v1);
	  //System.out.println("length " + length2);

	  double denominator = length1 * length2;

	  double product = denominator != 0.0 ? dotProduct / denominator : 0.0;

	  double angle = Math.acos (product);

	  return angle;
	}
	
	public static double computeAngle (double[] p0, double[] p1, double[] p2)
	{
	  double[] v0 = Geometry.createVector (p0, p1);
	  double[] v1 = Geometry.createVector (p0, p2);

	  //System.out.println("v0 x" + v0[0] + " y " + v0[1]);
	  //System.out.println("v1 x" + v1[0] + " y " + v1[1]);
	  
	  double dotProduct = Geometry.computeDotProduct (v0, v1);
	  //System.out.println("dot product " + dotProduct);

	  double length1 = Geometry.length (v0);
	  //System.out.println("length " + length1);
	  double length2 = Geometry.length (v1);
	  //System.out.println("length " + length2);

	  double denominator = length1 * length2;

	  double product = denominator != 0.0 ? dotProduct / denominator : 0.0;

	  double angle = Math.acos (product);

	  return angle;
	}
	
	public static Double distPtLine(PointF x1, PointF x2, PointF x0){
		double zaehler = Math.abs((x2.x-x1.x)*(x1.y-x0.y) - (x1.x-x0.x)*(x2.y-x1.y));
		double nenner = Math.sqrt(((x2.x - x1.x) * (x2.x - x1.x)) + ((x2.y - x1.y) * (x2.y - x1.y)));
		return zaehler/nenner;
	}
	
	public static double relativePosition(PointF lp1, PointF lp2, PointF point){
		Double[] v = {Double.valueOf(lp2.x - lp1.x), Double.valueOf(lp2.y - lp1.y)};
		double lineLength = magnitude(v);
		
		//double lineLength = lengthOfLine(line);
		
		double numerator = ((lp1.y - point.y) * (lp1.y - lp2.y)) - ((lp1.x - point.x)* (lp2.x - lp1.x)); 
		//float numerator = (me.getY(thumbIndex) - me.getY(thirdIndex)) * (me.getY(thumbIndex) - me.getY(pinkyIndex)) - (me.getX(thumbIndex) - me.getX(thirdIndex)) * (me.getX(pinkyIndex)-me.getX(thumbIndex));
    	return numerator / (lineLength * lineLength);
	}
	
	
	public static double computeSignedAngle(Double[] v1, Double[] v2){
		Double[] n_v1 = new Double[3];
		n_v1[0] = v1[0];
		n_v1[1] = v1[1];
		n_v1[2] = 0.0;
		Double[] n_v2 = new Double[3];
		n_v2[0] = v2[0];
		n_v2[1] = v2[1];
		n_v2[2] = 0.0;
		
		double angle = computeAngle(n_v1, n_v2);
		Double[] planeNormal = {0.0,0.0,1.0};
		//cross produce of vectors
		Double[] cross = Geometry.computeCrossProduct(n_v1, n_v2);
		double dot = scalarP(cross, planeNormal);
		if(dot < 0){
			return -angle;
		}
		return angle;
	}
	
	public static Float[] normalize(Float[] v){
		
		Float[] result = new Float[v.length];
		Float mag = magnitude(v);
		for(int i=0; i<v.length;i++){
			result[i] = v[i]/mag;
		}
		return result;
		
		
	}
	
public static Double[] normalize(Double[] v){
		
	Double[] result = new Double[v.length];
	Double mag = magnitude(v);
		for(int i=0; i<v.length;i++){
			result[i] = v[i]/mag;
		}
		return result;
		
		
	}
	

public static PointF intersects(PointF start1,
		Float[]v1, PointF start2, Float[] v2) {

	float x1 = start1.x;
	float y1 = start1.y;
	float x2 = x1 + v1[0];
	float y2 = y1 + v1[1];

	float x3 = start2.x;
	float y3 = start2.y;
	float x4 = x3 + v2[0];
	float y4 = y3 + v2[1];

	float d = ((x1 - x2) * (y3 - y4)) - ((y1 - y2) * (x3 - x4));
	if(d == 0) // If parallel, defaults to the average location of the lines.
	    return new PointF((x1 + x3) * 0.5f, (y1 + y3) * 0.5f);
	else
	{
	    float a = (x1 * y2) - (y1 * x2);
	    float b = (x3 * y4) - (y3 * x4);
	    return new PointF(((a * (x3 - x4)) - ((x1 - x2) * b)) / d, 
	                        ((a * (y3 - y4)) - ((y1 - y2) * b)) / d);
	}
}

	public static PointF rotatePoint(PointF b, PointF r, float angle){
		float[] rel_trans = {r.x, r.y};
		Matrix M_B_C = new Matrix();
		M_B_C.setRotate(-angle, b.x, b.y);
		//M_B_C.postRotate(-angle);
		M_B_C.mapPoints(rel_trans);
		
		return new PointF(rel_trans[0],rel_trans[1]);
	}
	
	public static PointF transformPoint(PointF p, PointF thumb, PointF finger){
		Double[] xAxis_C = {(double)finger.x - thumb.x, (double)finger.y - thumb.y};
	    Double[] xAxis = {1.0, 0.0};
		
		double angle = Utils.computeSignedAngle(xAxis, xAxis_C);
		
		//System.out.println(" angle " + Math.toDegrees(angle) + " angle radian " + angle) ;
		Matrix M_B_C = new Matrix();
		//if(angle < 0){
			//then turn clockwise
			//M_B_C = new Matrix(Math.cos(angle), -Math.sin(angle),Math.sin(angle), Math.cos(angle), (double)finger.x,(double)finger.y);
		M_B_C.postRotate((float)angle);
		M_B_C.postTranslate(finger.x, finger.y);
		//}else{
			//M_B_C = new AffineTransform(Math.cos(angle), Math.sin(angle),-Math.sin(angle), Math.cos(angle), finger.getX(),finger.getY());
			//System.out.println(M_B_C.toString());
			//Point2D.Double d = new Point2D.Double(1.060660171779821, 1.7677669529663689);
			//M_B_C.transform(d, d);
			//System.out.println("P_B? " + d.toString());
		//}
		
		Matrix T_M_B_C = new Matrix();
		
		M_B_C.invert(T_M_B_C);
		
		float[] point = {p.x, p.y};
		//float[] resultPoint = new float[2];
		
		T_M_B_C.mapPoints(point);
		PointF result = new PointF(point[0],point[1]);
		
		return result;
	}
	
	public static double computeAngle(Double[] v1, Double[] v2)
	{
		Double dotP = scalarP(v1, v2);
		Double v1_m = magnitude(v1);
		Double v2_m = magnitude(v2);
		
		return Math.acos(dotP/(v1_m * v2_m));
	}
	
	
public static ArrayList<String[]> getDataWithID(String ID,ArrayList<String[]> trial ){
		
		ArrayList<String[]> data = new ArrayList<String[]>();
		
		for(String[] t : trial){
			if(t[TOUCHID].equals(ID)){
				data.add(t);
			}
		}
		
		
		return data;
	}
	
	public static ArrayList<PointF> getTouchesWithID(String ID,ArrayList<String[]> trial ){
		
		ArrayList<PointF> touches = new ArrayList<PointF>();
		
		for(String[] t : trial){
			if(t[TOUCHID].equals(ID)){
				PointF touch = new PointF(java.lang.Float.valueOf(t[TOUCH_X]), java.lang.Float.valueOf(t[TOUCH_Y]));
				touches.add(touch);
			}
		}
		
		
		return touches;
	}
	
	
	
	
}

