package de.lmu.ifi.mfchords.views;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.PointF;
import android.graphics.drawable.Drawable;
import android.support.v4.view.MotionEventCompat;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import de.lmu.ifi.mfchords.R;
import de.lmu.ifi.mfchords.utils.Constants;
import de.lmu.ifi.mfchords.utils.MFCRecognizer;


public class CalibrationView extends LinearLayout {

	private ArrayList<Integer> pointer = new ArrayList<Integer>();
	@SuppressLint("UseSparseArrays")
	
	private MotionEvent motionEvent = null;
	
	private Drawable nf_index;
	private Drawable nf_middle;
	private Drawable nf_ring;
	private Drawable tp_index;
	private Drawable tp_middle;
	private Drawable tp_ring;
	private Drawable ti_middle;
	private Drawable ti_ring;
	private Drawable ti_pinky;
	
	
	private PointF first = new PointF();
	private PointF sec = new PointF();
	private PointF third = new PointF();
	
	
	private Drawable instruction;
	LinkedHashMap<String,Drawable> instructions = new LinkedHashMap< String,Drawable>();
	private int drawablePointer = 0;
	private int repetition = 3;
	
	
	private long twoFingersDownStamp = 0;
	private RelativeLayout rl;
	
	private ImageView instructionImage;
	private String currentInstruction;
	private TextView readyMsg;
	
	
	private MFCRecognizer recognizer = MFCRecognizer.getRecognizer();
	private Paint touchFeedbackPaint = new Paint();
	private Paint textPaint;
	
	
	@SuppressLint("NewApi")
	public CalibrationView(Context context) {
		super(context);
		// TODO Auto-generated constructor stub
		Log.v("myDebugTag", "calib view created");
		
		this.setWillNotDraw(false);
		
		touchFeedbackPaint.setColor(Color.CYAN);
		touchFeedbackPaint.setStyle(Style.FILL);
		touchFeedbackPaint.setStrokeWidth(15);
		
		nf_index = getContext().getResources().getDrawable(R.drawable.nf_index);
		instructions.put(Constants.NF_INDEX,nf_index);
		
		nf_middle = getContext().getResources().getDrawable(R.drawable.nf_middle);
		instructions.put(Constants.NF_MIDDLE,nf_middle);
		
		nf_ring = getContext().getResources().getDrawable(R.drawable.nf_ring);
		instructions.put(Constants.NF_RING,nf_ring);
		
		ti_middle = getContext().getResources().getDrawable(R.drawable.ti_middle);
		instructions.put(Constants.TI_MIDDLE,ti_middle);
		
		ti_ring = getContext().getResources().getDrawable(R.drawable.ti_ring);
		instructions.put(Constants.TI_RING,ti_ring);
		
		ti_pinky = getContext().getResources().getDrawable(R.drawable.ti_pinky);
		instructions.put(Constants.TI_PINKY,ti_pinky);
		
		tp_index = getContext().getResources().getDrawable(R.drawable.tp_index);
		instructions.put(Constants.TP_INDEX,tp_index);
		
		tp_middle = getContext().getResources().getDrawable(R.drawable.tp_middle);
		instructions.put(Constants.TP_MIDDLE,tp_middle);
		
		tp_ring = getContext().getResources().getDrawable(R.drawable.tp_ring);
		instructions.put(Constants.TP_RING,tp_ring);
		
				
		textPaint = new Paint();
        textPaint.setColor(Color.RED);
    	textPaint.setTextSize(30);
    	textPaint.setStyle(Paint.Style.FILL);
	    
	    
		LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		rl  = (RelativeLayout)inflater.inflate(R.layout.calib_layout, null);
		
		instructionImage = (ImageView)rl.findViewById(R.id.instruction);
		readyMsg = (TextView) rl.findViewById(R.id.finished);
		readyMsg.setVisibility(View.INVISIBLE);
		
		
		//iv.setImageDrawable(instructions.get(key))
		showNextInstruction();
		
		this.addView(rl);
		
		invalidate();
		
	}
	
	

	@Override
	protected void onLayout(boolean chagned, int l, int t, int r, int b) {
		// TODO Auto-generated method stub
		
		for(int i = 0 ; i < getChildCount() ; i++){
            getChildAt(i).layout(l, t, r, b);
            getChildAt(i).bringToFront();
        }
		
	}
	
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		
		canvas.drawColor(Color.WHITE);
		
		
		//Log.v("myDebugTag", "drawing " + recognizer.handModel);
		/*if(recognizer != null && recognizer.handModel != null){
			for(Finger f : recognizer.handModel.allFingers)
			{
				Log.v("myDebugTag", "drawing " + f.position.x + " " + f.position.y);
				//canvas.drawCircle(f.position.x, f.position.y - 50 , 50, touchFeedbackPaint);
			}
		}*/
		
		/*for(PointF p: points.values()) {
			canvas.drawCircle(p.x, p.y - 50 , 50, touchFeedbackPaint);
			
		}
		points.clear();
		
		
		canvas.drawText("FIRST", first.x, first.y-20, textPaint);
		//Log.v("myDebugTag", "drawing " + first.x + " " + first.y);
		canvas.drawText("SEC", sec.x, sec.y-20, textPaint);
		canvas.drawText("THIRD", third.x, third.y-20, textPaint);*/
		
	}
	
	
	
	private void showNextInstruction(){
		
		if(drawablePointer == instructions.size()){
			repetition--;
			drawablePointer = 0;
			if(repetition == 0){
				//all data is collected, now create classifier
				this.recognizer.createClassifier();
				instructionImage.setVisibility(View.INVISIBLE);
				readyMsg.setVisibility(View.VISIBLE);
				
				//write to file
				this.recognizer.writeCalibToFile();
				
				invalidate();
				return;
			}
		}
		
		if(drawablePointer < instructions.size() && repetition > 0){
			
			instruction = (new ArrayList<Drawable>(instructions.values())).get(drawablePointer);
			instructionImage.setImageDrawable(instruction);
			currentInstruction = this.getKeyForInstruction();
			drawablePointer++;
		}
		
		
		
		
		//instruction.setBounds(150,150, 450,450);
		invalidate();
		
	}
	
	
	
	@Override
    public boolean onTouchEvent(MotionEvent event) {
		motionEvent = event;
		final int action = MotionEventCompat.getActionMasked(event);
		int index = MotionEventCompat.getActionIndex(event);
    	int pointerID = MotionEventCompat.getPointerId(event, index);
		
    	
    	
    	if(!pointer.contains(Integer.valueOf(pointerID))){
			pointer.add(Integer.valueOf(pointerID));
		}
    	
    	
		switch (action) {
		
    		case MotionEvent.ACTION_POINTER_DOWN:
    				if(pointer.size() == 2){
    					twoFingersDownStamp = System.currentTimeMillis();
    				}else if(pointer.size() == 3){
    					ArrayList<PointF> pointerList = new ArrayList<PointF>();
    					for(Integer i : pointer){
    						//feed the classifier
    						pointerList.add(new PointF(MotionEventCompat.getX(event, i),MotionEventCompat.getY(event, i)));
    					}
    					
    					boolean isTwoStepPosture = true;
    					if((System.currentTimeMillis() - twoFingersDownStamp) < 100){
    						isTwoStepPosture = false;
    					}
    					
    					
    					
    					if(!this.recognizer.isClassifierBuilt()){
    						this.recognizer.feedClassifier(this.recognizer.processPoints(pointerList,isTwoStepPosture), currentInstruction);
    						
    					}/*else{
    						String predicted = this.recognizer.getClassifiedMFC(this.recognizer.processPoints(pointerList,isTwoStepPosture));
    						Log.v("myDebugTag", "predicted " + predicted);
    						instructionImage.setVisibility(View.VISIBLE);
    						readyMsg.setVisibility(View.INVISIBLE);
    						Drawable d = instructions.get(predicted);
    						if(d != null){
    							instructionImage.setImageDrawable(instruction);
    						}
    					}*/
    					
    					showNextInstruction();
    				}
    				invalidate();
    				break;
    		case MotionEvent.ACTION_UP:
        		if(pointer.contains(Integer.valueOf(pointerID))){
        			pointer.remove(Integer.valueOf(pointerID));
        			
        		}
        		invalidate();
        		break;
    		case MotionEvent.ACTION_POINTER_UP:
        		if(pointer.contains(Integer.valueOf(pointerID))){
        			pointer.remove(Integer.valueOf(pointerID));
        		}
        		break;
		}
		return true;
	}
	
	
	
	
	
	private String getKeyForInstruction(){
		for(String k: instructions.keySet()){
			if(instructions.get(k).equals(instruction)){
				return k;
			}
		}
		return null;
	}
	
	
	protected void dispatchDraw(Canvas canvas) {
		//canvas.drawColor(Color.YELLOW);
		super.dispatchDraw(canvas);
		
		
		
		
		
	}
	
	
}
